@@ -62,7 +62,11 @@ from meadowrun.deployment_spec import (
     PreinstalledInterpreter,
     Secret,
 )
-from meadowrun.k8s_integration.k8s import Kubernetes
+from meadowrun.k8s_integration.k8s import Kubernetes, StorageBucketSpec
+from meadowrun.k8s_integration.storage_spec import (
+    GenericStorageBucketSpec,
+    GoogleBucketSpec,
+)
 from meadowrun.run_job import (
     TaskResult,
     run_command,
@@ -79,7 +83,7 @@ from meadowrun.run_job_core import (
     TaskException,
 )
 from meadowrun.run_job_core import Host
-from meadowrun.run_job_local import LocalHost, MACHINE_CACHE_FOLDER
+from meadowrun.run_job_local import MACHINE_CACHE_FOLDER
 from meadowrun.version import __version__
 
 __all__ = [
@@ -109,11 +113,13 @@ __all__ = [
     "PreinstalledInterpreter",
     "Secret",
     "Host",
-    "LocalHost",
     "MACHINE_CACHE_FOLDER",
     "Resources",
     "SshHost",
     "Kubernetes",
+    "StorageBucketSpec",
+    "GenericStorageBucketSpec",
+    "GoogleBucketSpec",
     "RunMapTasksFailedException",
     "TaskResult",
     "TaskException",
