@@ -61,7 +61,6 @@ from meadowrun.aws_integration.management_lambdas.ec2_alloc_stub import (
     _RUNNING_JOBS,
     ignore_boto3_error_code,
 )
-from meadowrun.aws_integration.s3 import S3ObjectStorage, _get_bucket_name
 from meadowrun.instance_allocation import (
     InstanceRegistrar,
     _InstanceState,
@@ -77,16 +76,19 @@ from meadowrun.run_job_core import (
     SshHost,
     WaitOption,
 )
-from meadowrun.s3_grid_job import receive_results
+from meadowrun.storage_grid_job import (
+    S3Bucket,
+    get_aws_s3_bucket,
+    receive_results,
+)
 
 if TYPE_CHECKING:
     from types import TracebackType
 
+    from meadowrun.abstract_storage_bucket import AbstractStorageBucket
     from meadowrun.meadowrun_pb2 import Job
-    from meadowrun.object_storage import ObjectStorage
     from meadowrun.run_job_core import TaskProcessState, WorkerProcessState
     from meadowrun.run_job_local import TaskWorkerServer, WorkerMonitor
-    from types_aiobotocore_s3 import S3Client
     from types_aiobotocore_sqs import SQSClient
     from typing_extensions import Literal
 
@@ -100,26 +102,26 @@ _U = TypeVar("_U")
 # replicate into each region.
 _AMIS = {
     "plain": {
-        "us-east-2": "ami-02cb932ead54f2c81",
-        "us-east-1": "ami-0c4f30064b16cac99",
-        "us-west-1": "ami-012fbb53effdd0f6f",
-        "us-west-2": "ami-039fa3adc59d9335b",
-        "eu-central-1": "ami-0f920323773495495",
-        "eu-west-1": "ami-037a59c4c352cfe48",
-        "eu-west-2": "ami-0e768a7fdd3b895fe",
-        "eu-west-3": "ami-0f46081510e4539ff",
-        "eu-north-1": "ami-02d2946f47e774859",
+        "us-east-2": "ami-01fca960c9299b335",
+        "us-east-1": "ami-03605d9958c76f9ed",
+        "us-west-1": "ami-0f4875d5d9c68dff4",
+        "us-west-2": "ami-0e6855d6badbfd1bb",
+        "eu-central-1": "ami-00d7b0f1c05591b34",
+        "eu-west-1": "ami-0e6989e6e25e17893",
+        "eu-west-2": "ami-008e7c6aecbbcd3e9",
+        "eu-west-3": "ami-0b2ab4237804283b3",
+        "eu-north-1": "ami-0d41c6161b3bdf6b9",
     },
     "cuda": {
-        "us-east-2": "ami-03c47749b6700ef26",
-        "us-east-1": "ami-01884f83d0fd5cad4",
-        "us-west-1": "ami-0ce1ef3db8dd5cb59",
-        "us-west-2": "ami-08f11c1824838d2c6",
-        "eu-central-1": "ami-01e7286e8f0e33e45",
-        "eu-west-1": "ami-08d3ed38be713e3c2",
-        "eu-west-2": "ami-0f0eaaa2947ac76ce",
-        "eu-west-3": "ami-059fc114d6efb01bf",
-        "eu-north-1": "ami-09279ba9a41266369",
+        "us-east-2": "ami-0da59419ffa313be5",
+        "us-east-1": "ami-0c2990d11681b3c3c",
+        "us-west-1": "ami-0e7675039071f6674",
+        "us-west-2": "ami-0811b0dd8b334d076",
+        "eu-central-1": "ami-0706e0cc9199ea6a3",
+        "eu-west-1": "ami-0d939ea5b21e04a2c",
+        "eu-west-2": "ami-0bd66e6a4b8dbd3e2",
+        "eu-west-3": "ami-0ac87c5282b090f4e",
+        "eu-north-1": "ami-0f663e908bc88c42c",
     },
 }
 SSH_USER = "ubuntu"
@@ -186,9 +188,9 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
 
     async def __aexit__(
         self,
-        exc_typ: Type[BaseException],
-        exc_val: BaseException,
-        exc_tb: TracebackType,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
     ) -> None:
         pass
 
@@ -680,8 +682,8 @@ class AllocEC2Instance(AllocVM):
     def _create_grid_job_cloud_interface(self) -> GridJobCloudInterface:
         return EC2GridJobInterface(self)
 
-    async def get_object_storage(self) -> ObjectStorage:
-        return S3ObjectStorage(self._get_region_name())
+    async def get_storage_bucket(self) -> AbstractStorageBucket:
+        return get_aws_s3_bucket(self._get_region_name())
 
 
 async def run_job_ec2_instance_registrar(
@@ -735,28 +737,26 @@ class EC2GridJobInterface(GridJobCloudInterface):
         self._job_id = str(uuid.uuid4())
 
         self._sqs_client: Optional[SQSClient] = None
-        self._s3_client: Optional[S3Client] = None
+        self._s3_bucket: Optional[S3Bucket] = None
 
     async def __aenter__(self) -> EC2GridJobInterface:
         session = aiobotocore.session.get_session()
         self._sqs_client = await session.create_client(
             "sqs", region_name=self._region_name
         ).__aenter__()
-        self._s3_client = await session.create_client(
-            "s3", region_name=self._region_name
-        ).__aenter__()
+        self._s3_bucket = await get_aws_s3_bucket(self._region_name).__aenter__()
         return self
 
     async def __aexit__(
         self,
-        exc_typ: Type[BaseException],
-        exc_val: BaseException,
-        exc_tb: TracebackType,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
     ) -> None:
         if self._sqs_client is not None:
-            await self._sqs_client.__aexit__(exc_typ, exc_val, exc_tb)
-        if self._s3_client is not None:
-            await self._s3_client.__aexit__(exc_typ, exc_val, exc_tb)
+            await self._sqs_client.__aexit__(exc_type, exc_val, exc_tb)
+        if self._s3_bucket is not None:
+            await self._s3_bucket.__aexit__(exc_type, exc_val, exc_tb)
 
     def create_instance_registrar(self) -> InstanceRegistrar:
         return EC2InstanceRegistrar(self._region_name, "create")
@@ -776,7 +776,7 @@ class EC2GridJobInterface(GridJobCloudInterface):
         return index
 
     async def setup_and_add_tasks(self, tasks: Sequence[_T]) -> None:
-        if self._sqs_client is None or self._s3_client is None:
+        if self._sqs_client is None or self._s3_bucket is None:
             raise ValueError("EC2GridJobInterface must be created with `async with`")
 
         # create SQS queues and add tasks to the request queue
@@ -791,7 +791,7 @@ class EC2GridJobInterface(GridJobCloudInterface):
             add_tasks(
                 self._job_id,
                 await self._request_queue_urls[queue_index],
-                self._s3_client,
+                self._s3_bucket,
                 self._sqs_client,
                 tasks,
             )
@@ -835,7 +835,7 @@ class EC2GridJobInterface(GridJobCloudInterface):
     async def receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
     ) -> AsyncIterable[Tuple[List[TaskProcessState], List[WorkerProcessState]]]:
-        if self._s3_client is None:
+        if self._s3_bucket is None:
             raise ValueError("EC2GridJobInterface must be created with `async with`")
 
         # Note: download here is all via S3. It's important that downloads are fast
@@ -851,8 +851,7 @@ class EC2GridJobInterface(GridJobCloudInterface):
         # bucket.
 
         return receive_results(
-            self._s3_client,
-            _get_bucket_name(self._region_name),
+            self._s3_bucket,
             self._job_id,
             stop_receiving=stop_receiving,
             all_workers_exited=workers_done,
