@@ -9,6 +9,7 @@ import os
 import shutil
 import tempfile
 import urllib.parse
+import zipfile
 from typing import (
     Awaitable,
     Callable,
@@ -16,12 +17,14 @@ from typing import (
     Mapping,
     Optional,
     Sequence,
+    TYPE_CHECKING,
     Tuple,
     Union,
 )
 
 import filelock
 
+import meadowrun.func_worker_storage_helper
 from meadowrun._vendor.aiodocker import exceptions as aiodocker_exceptions
 from meadowrun.aws_integration.ecr import (
     get_ecr_helper,
@@ -30,8 +33,14 @@ from meadowrun.aws_integration.management_lambdas.ec2_alloc_stub import (
     _MEADOWRUN_GENERATED_DOCKER_REPO,
 )
 from meadowrun.azure_integration.acr import get_acr_helper
-from meadowrun.deployment.conda import get_cached_or_create_conda_environment
 from meadowrun.credentials import RawCredentials, SshKey
+from meadowrun.deployment.conda import get_cached_or_create_conda_environment
+from meadowrun.deployment.pip import get_cached_or_create_pip_environment
+from meadowrun.deployment.poetry import get_cached_or_create_poetry_environment
+from meadowrun.deployment.prerequisites import (
+    EnvironmentSpecPrerequisites,
+    GOOGLE_AUTH_PACKAGE,
+)
 from meadowrun.docker_controller import (
     _does_digest_exist_locally,
     build_image,
@@ -47,26 +56,17 @@ from meadowrun.meadowrun_pb2 import (
     ServerAvailableContainer,
     ServerAvailableInterpreter,
 )
-from meadowrun.deployment.pip import get_cached_or_create_pip_environment
-from meadowrun.deployment.poetry import (
-    get_cached_or_create_poetry_environment,
-)
-from meadowrun.azure_integration.blob_storage import AzureBlobStorage
-from meadowrun.aws_integration.s3 import S3ObjectStorage
 from meadowrun.run_job_core import (
     CloudProviderType,
     ContainerRegistryHelper,
 )
-from meadowrun.s3_grid_job import (
-    try_get_storage_file,
-    write_storage_file,
-)
-from meadowrun.func_worker_storage_helper import FuncWorkerClientObjectStorage
-import meadowrun.func_worker_storage_helper
-from meadowrun.deployment.prerequisites import (
-    EnvironmentSpecPrerequisites,
-    GOOGLE_AUTH_PACKAGE,
-)
+from meadowrun.storage_grid_job import download_chunked_file
+from meadowrun.abstract_storage_bucket import AbstractStorageBucket
+
+if TYPE_CHECKING:
+    StorageBucketFactoryType = Union[
+        AbstractStorageBucket, Callable[[], Awaitable[AbstractStorageBucket]], None
+    ]
 
 _GIT_REPO_URL_SUFFIXES_TO_REMOVE = [".git", "/"]
 
@@ -137,6 +137,7 @@ async def get_code_paths(
     local_copies_folder: str,
     job: Job,
     credentials: Optional[RawCredentials],
+    storage_bucket_factory: StorageBucketFactoryType,
 ) -> Tuple[Sequence[str], Optional[str], Optional[str]]:
     """
     Returns [code_paths, interpreter_spec_path, current_working_directory] based on
@@ -187,7 +188,9 @@ async def get_code_paths(
             credentials,
         )
     elif case == "code_zip_file":
-        return await _get_zip_file_code_paths(local_copies_folder, job.code_zip_file)
+        return await _get_zip_file_code_paths(
+            local_copies_folder, job.code_zip_file, storage_bucket_factory
+        )
     else:
         raise ValueError(f"Unrecognized code_deployment {case}")
 
@@ -262,32 +265,45 @@ async def _get_git_code_paths(
             shutil.copytree(local_path, local_copy_path)
 
         # TODO raise a friendlier exception if this path doesn't exist
-        source_path = os.path.join(local_copy_path, path_to_source)
+        source_path = os.path.realpath((os.path.join(local_copy_path, path_to_source)))
         return ([source_path], local_copy_path, local_copy_path)
 
 
-_ALL_OBJECT_STORAGES = {
-    # see the note on get_url_scheme
-    s.get_url_scheme(): s  # type: ignore
-    for s in (
-        AzureBlobStorage,
-        FuncWorkerClientObjectStorage,
-        S3ObjectStorage,
-    )
-}
-
-
 async def _get_zip_file_code_paths(
-    local_copies_folder: str, code_zip_file: CodeZipFile
+    local_copies_folder: str,
+    code_zip_file: CodeZipFile,
+    storage_bucket_factory: StorageBucketFactoryType,
 ) -> Tuple[List[str], None, str]:
+    """
+    code_zip_file.url will be the URL of a file in the object storage system as
+    generated by _upload_code_zip_file. This function should download the file and
+    extract it to local_copies_folder if it has not already been extracted.
+    """
     decoded_url = urllib.parse.urlparse(code_zip_file.url)
-    if decoded_url.scheme not in _ALL_OBJECT_STORAGES:
-        raise ValueError(f"Unknown URL scheme in {code_zip_file.url}")
+    if decoded_url.scheme != "mdrstorage":  # just a sanity check
+        raise ValueError(f"Unexpected URL scheme in {code_zip_file.url}")
 
-    async with _ALL_OBJECT_STORAGES[decoded_url.scheme]() as object_storage:
-        extracted_folder = await object_storage.download_and_unzip(
-            code_zip_file.url, local_copies_folder
-        )
+    object_name = decoded_url.path.lstrip("/")
+    extracted_folder = os.path.join(local_copies_folder, os.path.basename(object_name))
+
+    with filelock.FileLock(f"{extracted_folder}.lock", timeout=120):
+        if not os.path.exists(extracted_folder):
+            zip_file_path = extracted_folder + ".zip"
+            if storage_bucket_factory is None:
+                raise ValueError(
+                    "storage_bucket_factory must be set to download CodeZipFiles"
+                )
+            elif isinstance(storage_bucket_factory, AbstractStorageBucket):
+                await download_chunked_file(
+                    storage_bucket_factory, object_name, zip_file_path
+                )
+            else:
+                async with await storage_bucket_factory() as storage_bucket:
+                    await download_chunked_file(
+                        storage_bucket, object_name, zip_file_path
+                    )
+            with zipfile.ZipFile(zip_file_path) as zip_file:
+                zip_file.extractall(extracted_folder)
 
     return (
         [
@@ -480,31 +496,25 @@ async def compile_environment_spec_locally(
             f"Unexpected environment_type {environment_spec.environment_type}"
         )
 
+    # TODO this isn't the right way to do this--the storage client should be
+    # getting passed in from higher up. For now this works because this code
+    # path is only being hit from Kubernetes
+    storage_bucket = meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_BUCKET
+    if storage_bucket is None:
+        raise ValueError("FUNC_WORKER_STORAGE_BUCKET was not set")
     return ServerAvailableInterpreter(
         interpreter_path=await get_cached_or_create(
             spec_hash,
             path_to_spec,
             prerequisites,
             os.path.join(built_interpreters_folder, spec_hash),
-            # TODO this isn't the right way to do this--these try_get_storage_file and
-            # write_storage file functions should be getting passed in from higher up.
-            # For now this works because this code path is only being hit from
-            # Kubernetes
             # try_get_file(remote_file_name: str, local_file_name: str) -> bool. Tries
             # to download the specified remote file to the specified local file name.
             # Returns True if the file is available, False if the file is not available.
-            functools.partial(
-                try_get_storage_file,
-                meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_CLIENT,
-                meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_BUCKET,
-            ),
+            functools.partial(storage_bucket.try_get_file),
             # upload_file(local_file_name: str, remote_file_name: str) -> None. Uploads
             # the specified file, overwrites any existing remote file.
-            functools.partial(
-                write_storage_file,
-                meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_CLIENT,
-                meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_BUCKET,
-            ),
+            functools.partial(storage_bucket.write_file),
         )
     )
 
