@@ -1,23 +1,25 @@
 from __future__ import annotations
 
 import abc
+import argparse
 import asyncio
 import dataclasses
 import math
 import os
 import pickle
 import platform
+import shlex
 import time
 import uuid
 from typing import (
     TYPE_CHECKING,
     Any,
-    AsyncContextManager,
     AsyncIterable,
     Awaitable,
     Callable,
     Coroutine,
     Dict,
+    Iterable,
     List,
     Optional,
     Sequence,
@@ -35,13 +37,7 @@ import meadowrun.func_worker_storage_helper
 from meadowrun.config import GPU, LOGICAL_CPU, MEMORY_GB, MEADOWRUN_INTERPRETER
 from meadowrun.credentials import KubernetesSecretRaw
 from meadowrun.docker_controller import expand_ports
-from meadowrun.func_worker_storage_helper import (
-    MEADOWRUN_STORAGE_PASSWORD,
-    MEADOWRUN_STORAGE_USERNAME,
-    FuncWorkerClientObjectStorage,
-)
 from meadowrun.k8s_integration.k8s_core import (
-    get_kubernetes_secret,
     get_main_container_is_ready,
     get_pods_for_job,
     run_command_on_pod,
@@ -49,15 +45,6 @@ from meadowrun.k8s_integration.k8s_core import (
     wait_for_pod,
     wait_for_pod_running,
 )
-from meadowrun.s3_grid_job import (
-    complete_task,
-    download_task_arg,
-    get_job_completion_from_process_state,
-    get_storage_client_from_args,
-    read_storage,
-    receive_results,
-    upload_task_args,
-)
 from meadowrun.meadowrun_pb2 import (
     Job,
     ProcessState,
@@ -80,7 +67,13 @@ from meadowrun.run_job_local import (
     _string_pairs_to_dict,
     restart_worker,
 )
-from meadowrun.shared import none_async_context
+from meadowrun.storage_grid_job import (
+    complete_task,
+    download_task_arg,
+    get_job_completion_from_process_state,
+    receive_results,
+    upload_task_args,
+)
 from meadowrun.storage_keys import (
     storage_key_job_to_run,
     storage_key_ranges,
@@ -90,10 +83,9 @@ from meadowrun.version import __version__
 
 if TYPE_CHECKING:
     from asyncio import Task
-    import types_aiobotocore_s3
 
+    from meadowrun.abstract_storage_bucket import AbstractStorageBucket
     from meadowrun.instance_selection import ResourcesInternal
-    from meadowrun.object_storage import ObjectStorage
     from meadowrun.run_job_core import WorkerProcessState, TaskProcessState
 
 MEADOWRUN_POD_NAME = "MEADOWRUN_POD_NAME"
@@ -107,7 +99,6 @@ async def _indexed_map_worker(
     total_num_tasks: int,
     num_workers: int,
     job_id: str,
-    result_highest_pickle_protocol: int,
     worker_server: TaskWorkerServer,
     worker_monitor: WorkerMonitor,
 ) -> None:
@@ -115,7 +106,7 @@ async def _indexed_map_worker(
     This is a worker function to help with running a run_map. This worker assumes that
     JOB_COMPLETION_INDEX is set, which Kubernetes will set for indexed completion jobs.
     This worker assumes task arguments are accessible via
-    meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_CLIENT and will just
+    meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_BUCKET and will just
     complete all of the tasks where task_index % num_workers == current worker index.
     """
 
@@ -125,24 +116,15 @@ async def _indexed_map_worker(
         os.environ.get("MEADOWRUN_WORKER_INDEX", os.environ["JOB_COMPLETION_INDEX"])
     )
 
-    storage_username = os.environ.get(MEADOWRUN_STORAGE_USERNAME, None)
-    storage_password = os.environ.get(MEADOWRUN_STORAGE_PASSWORD, None)
-    if storage_username is None and storage_password is None:
-        raise ValueError("Cannot call _indexed_map_worker without a storage client")
-
     # we're always being called from run_job_local_storage_main which sets these
     # variables for us
-    storage_client = meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_CLIENT
     storage_bucket = meadowrun.func_worker_storage_helper.FUNC_WORKER_STORAGE_BUCKET
-    if storage_client is None or storage_bucket is None:
+    if storage_bucket is None:
         raise ValueError(
             "Programming error--_indexed_map_worker must be called from "
             "run_job_local_storage_main"
         )
 
-    result_pickle_protocol = min(
-        result_highest_pickle_protocol, pickle.HIGHEST_PROTOCOL
-    )
     # only the first option (MEADOWRUN_POD_NAME) will give the actual name of the pod.
     # The hostname is similar but slightly different, and we'll fall back on that if
     # something has gone wrong and MEADOWRUN_POD_NAME is not available
@@ -152,26 +134,24 @@ async def _indexed_map_worker(
     log_file_name = f"{pod_name}:/var/meadowrun/job_logs/{job_id}.log"
 
     byte_ranges = pickle.loads(
-        await read_storage(storage_client, storage_bucket, storage_key_ranges(job_id))
+        await storage_bucket.get_bytes(storage_key_ranges(job_id))
     )
 
     i = current_worker_index
     while i < total_num_tasks:
-        arg = await download_task_arg(
-            storage_client, storage_bucket, job_id, byte_ranges[i]
-        )
+        arg = await download_task_arg(storage_bucket, job_id, byte_ranges[i])
 
         try:
             worker_monitor.start_stats()
             await worker_server.send_message(arg)
-            state, result = await worker_server.receive_message()
+            state, result_bytes = await worker_server.receive_message()
             stats = await worker_monitor.stop_stats()
 
             process_state = ProcessState(
                 state=ProcessState.ProcessStateEnum.SUCCEEDED
                 if state == "SUCCEEDED"
                 else ProcessState.ProcessStateEnum.PYTHON_EXCEPTION,
-                pickled_result=pickle.dumps(result, protocol=result_pickle_protocol),
+                pickled_result=result_bytes,
                 return_code=0,
                 max_memory_used_gb=stats.max_memory_used_gb,
                 log_file_name=log_file_name,
@@ -187,7 +167,7 @@ async def _indexed_map_worker(
             await restart_worker(worker_server, worker_monitor)
 
         # we don't support retries yet so we're always on attempt 1
-        await complete_task(storage_client, storage_bucket, job_id, i, 1, process_state)
+        await complete_task(storage_bucket, job_id, i, 1, process_state)
 
         print(
             f"Meadowrun agent: Completed task #{i}, "
@@ -287,6 +267,49 @@ async def _image_name_from_job(job: Job) -> Tuple[bool, str, Optional[str], Job]
     return is_custom_container_image, image_name, image_pull_secret_name, modified_job
 
 
+class StorageBucketSpec(abc.ABC):
+    """
+    An abstract class that specifies an object storage system that Meadowrun can use to
+    send data back and forth from remote workers
+    """
+
+    @abc.abstractmethod
+    async def get_storage_bucket(
+        self, kubernetes_namespace: str
+    ) -> AbstractStorageBucket:
+        ...
+
+    @abc.abstractmethod
+    def get_command_line_arguments(self) -> List[str]:
+        # Returns command line arguments that can be parsed via add_arguments_to_parser
+        # and from_parsed_args to re-create the storage spec in a remote process
+        ...
+
+    @abc.abstractmethod
+    def get_environment_variables(self) -> Iterable[kubernetes_client.V1EnvVar]:
+        # Environment variables returned here will be added to the pod spec
+        ...
+
+    @classmethod
+    @abc.abstractmethod
+    def get_storage_type(cls) -> str:
+        # This is used together with get_command_line_arguments so that a remote process
+        # knows what sub-class of StorageBucketSpec to use to re-create the storage spec
+        ...
+
+    @classmethod
+    @abc.abstractmethod
+    def add_arguments_to_parser(cls, parser: argparse.ArgumentParser) -> None:
+        # See get_command_line_arguments
+        ...
+
+    @classmethod
+    @abc.abstractmethod
+    async def from_parsed_args(cls, args: argparse.Namespace) -> AbstractStorageBucket:
+        # See get_command_line_arguments
+        ...
+
+
 @dataclasses.dataclass(frozen=True)
 class Kubernetes(Host):
     """
@@ -294,108 +317,51 @@ class Kubernetes(Host):
     optional with the Kubernetes Host.
 
     Attributes:
-        storage_bucket: Together, the storage_* arguments specify an S3-compatible
-            object store that Meadowrun will use for sending inputs/outputs from the
-            Kubernetes job and back. For run_command, the S3-compatible object store is
-            not used (even if it is specified). For run_function on a pickled function
-            (e.g. a reference to a python function or a lambda) or where args/kwargs is
-            specified, the object store is required, which means that storage_bucket
-            must be specified. For run_function on a string referencing a function, the
-            object store is optional, which means that if storage_bucket is provided,
-            then it will be used to return results. If storage_bucket is not provided,
-            the result of the function will not be available.
-
-            In all cases, if storage_bucket is None, all of the other storage_*
-            arguments will be ignored.
-
-            Together, the storage_* arguments should be configured so that:
-
-            ```python
-            import boto3
-            boto3.Session(
-                aws_access_key_id=storage_username,
-                aws_secret_access_key=storage_password
-            ).client(
-                "s3", endpoint_url=storage_endpoint_url
-            ).download_file(
-                Bucket=storage_bucket, Key="test.file", Filename="test.file"
-            )
-            ```
-
-            works. `storage_username` and `storage_password` should be the values
-            provided by `storage_username_password_secret`. (boto3 is built to be used
-            with AWS S3, but it should work with any S3-compatible object store like
-            Minio, Ceph, etc.)
-        storage_file_prefix: Part of the specification of the S3-compatible object store
-            to use (see storage_bucket). The prefix will be used when naming the
-            Meadowrun-generated files in the object store. Defaults to "". For non empty
-            string values, this should usually end with "/". If you leave off the "/",
-            the job_id will just be concatenated to the string, e.g. "foo12345..."
-            rather than "foo/12345"
-        storage_endpoint_url: Part of the specification of the S3-compatible object
-            store to use (see storage_bucket).
-        storage_endpoint_url_in_cluster: Part of the specification of the S3-compatible
-            object store to use (see storage_bucket). Defaults to None which means use
-            storage_endpoint_url. You can set this to a different URL if you need to use
-            a different URL from inside the Kubernetes cluster to access the storage
-            endpoint
-        storage_username_password_secret: Part of the specification of the S3-compatible
-            object store to use (see storage_bucket). This should be the name of a
-            Kubernetes secret that has a "username" and "password" key, where the
-            username and password can be used to authenticate with the storage API.
+        storage_spec: Specifies the object storage system to use. See derived classes of
+            [StorageBucketSpec][meadowrun.StorageBucketSpec]. This can only be omitted
+            if you are only using run_command and you've specified a specific container
+            image to run on (i.e. rather than an EnvironmentSpec of some sort)
         kube_config_context: Specifies the kube config context to use. Default is None
             which means use the current context (i.e. `kubectl config current-context`)
         kubernetes_namespace: The Kubernetes namespace that Meadowrun will create Jobs
             in. This should usually not be left to the default value ("default") for any
             "real" workloads.
-        resuable_pods: Experimental feature: rather than always starting a new pod for
-            each job, starts generic long-lived pods that can be reused for multiple
-            jobs
+        resuable_pods: When set to True, starts generic long-lived pods that can be
+            reused for multiple jobs. When set to False, starts a new pod(s) for every
+            job
+        pod_customization: A function like pod_customization(pod_template_spec) that
+            will be called on the PodTemplateSpec just before we submit it to
+            Kubernetes. You can make changes like specifying a serviceAccountName,
+            adding ephemeral storage, etc. You can either modify the pod_template_spec
+            argument in place and return it as the result, or construct a new
+            [V1PodTemplateSpec](https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1PodTemplateSpec.md)
+            and return that.
     """
 
-    storage_bucket: Optional[str] = None
-    storage_endpoint_url: Optional[str] = None
-    storage_endpoint_url_in_cluster: Optional[str] = None
-    storage_username_password_secret: Optional[str] = None
+    storage_spec: Optional[StorageBucketSpec]
     kube_config_context: Optional[str] = None
     kubernetes_namespace: str = "default"
-    reusable_pods: bool = False
+    reusable_pods: bool = True
     pod_customization: Optional[
         Callable[
             [kubernetes_client.V1PodTemplateSpec], kubernetes_client.V1PodTemplateSpec
         ]
     ] = None
 
-    def get_storage_endpoint_url_in_cluster(self) -> Optional[str]:
-        if self.storage_endpoint_url_in_cluster is not None:
-            return self.storage_endpoint_url_in_cluster
-        return self.storage_endpoint_url
-
-    async def _get_storage_client(
-        self,
-    ) -> AsyncContextManager[Optional[types_aiobotocore_s3.S3Client]]:
+    async def get_storage_bucket(self) -> AbstractStorageBucket:
         # get the storage client
 
         # this is kind of weird, this should be called before any Kubernetes function,
-        # but for now, _get_storage_client is always the first thing that's called
+        # but for now, _get_storage_bucket is always the first thing that's called
         await kubernetes_config.load_kube_config(context=self.kube_config_context)
 
-        if self.storage_bucket is not None:
-            if self.storage_username_password_secret is not None:
-                secret_data = await get_kubernetes_secret(
-                    self.kubernetes_namespace,
-                    self.storage_username_password_secret,
-                )
-            else:
-                secret_data = {}
-
-            return get_storage_client_from_args(
-                self.storage_endpoint_url,
-                secret_data.get("username", None),
-                secret_data.get("password", None),
+        if self.storage_spec is None:
+            raise ValueError(
+                "The functionality you are trying to use requires specifying a "
+                "storage_bucket. Please specify a storage_spec."
             )
 
-        return none_async_context()
+        return await self.storage_spec.get_storage_bucket(self.kubernetes_namespace)
 
     async def run_job(
         self,
@@ -497,7 +463,7 @@ class Kubernetes(Host):
                     image_name,
                     command,
                     environment_variables,
-                    self.storage_username_password_secret,
+                    self.storage_spec,
                     image_pull_secret_name,
                     1,
                     [int(p) for p in expand_ports(job.ports)],
@@ -546,7 +512,7 @@ class Kubernetes(Host):
         wait_for_result: WaitOption,
     ) -> JobCompletion[Any]:
         # This is the "normal" way to run jobs on Kubernetes. We assume that the image
-        # specified by image_name has Meadowrun set up in it. We use the storage_client
+        # specified by image_name has Meadowrun set up in it. We use the storage_bucket
         # to send a Job object via S3-compatible object storage and run
         # run_job_local_storage_main which will actually run our job. The results will
         # also come back via the object storage.
@@ -560,30 +526,19 @@ class Kubernetes(Host):
         else:
             process = SingleUsePodRemoteProcesses()
 
-        async with await self._get_storage_client() as storage_client, kubernetes_client.ApiClient() as api_client, kubernetes_stream.WsApiClient() as ws_api_client:  # noqa: E501
+        async with await self.get_storage_bucket() as storage_bucket, kubernetes_client.ApiClient() as api_client, kubernetes_stream.WsApiClient() as ws_api_client:  # noqa: E501
+            # just for mypy, get_storage_bucket requires this
+            assert self.storage_spec is not None
+
             core_api = kubernetes_client.CoreV1Api(api_client)
             ws_core_api = kubernetes_client.CoreV1Api(ws_api_client)
             batch_api = kubernetes_client.BatchV1Api(api_client)
 
             try:
-                storage_endpoint_url = self.get_storage_endpoint_url_in_cluster()
-                if (
-                    storage_client is None
-                    or self.storage_bucket is None
-                    or storage_endpoint_url is None
-                ):
-                    raise ValueError(
-                        "Cannot use an environment_spec without providing a "
-                        "storage_bucket. Please either provide a pre-built container "
-                        "image for the interpreter or provide a storage_bucket"
-                    )
-
                 job_to_run_key = storage_key_job_to_run(job.job_id)
                 storage_keys_used.append(job_to_run_key)
-                await storage_client.put_object(
-                    Bucket=self.storage_bucket,
-                    Key=job_to_run_key,
-                    Body=job.SerializeToString(),
+                await storage_bucket.write_bytes(
+                    job.SerializeToString(), job_to_run_key
                 )
 
                 try:
@@ -597,9 +552,7 @@ class Kubernetes(Host):
                             image_pull_secret_name,
                             [int(p) for p in expand_ports(job.ports)],
                             resources_required,
-                            storage_endpoint_url,
-                            self.storage_bucket,
-                            self.storage_username_password_secret,
+                            self.storage_spec,
                             job.job_id,
                             1,
                             wait_for_result,
@@ -642,8 +595,7 @@ class Kubernetes(Host):
                 if job_spec_type is None:
                     raise ValueError("Unexpected, job.job_spec is None")
                 result = await get_job_completion_from_process_state(
-                    storage_client,
-                    self.storage_bucket,
+                    storage_bucket,
                     job.job_id,
                     "0",
                     job_spec_type,
@@ -656,12 +608,10 @@ class Kubernetes(Host):
                 await process.kill_all()
                 # TODO we should separately periodically clean up these files in case we
                 # aren't able to execute this finally block
-                if storage_client is not None and self.storage_bucket is not None:
+                if storage_bucket is not None:
                     await asyncio.gather(
                         *(
-                            storage_client.delete_object(
-                                Bucket=self.storage_bucket, Key=storage_key
-                            )
+                            storage_bucket.delete_object(storage_key)
                             for storage_key in storage_keys_used
                         ),
                         return_exceptions=True,
@@ -687,17 +637,10 @@ class Kubernetes(Host):
         if max_num_task_attempts != 1:
             raise NotImplementedError("max_num_task_attempts must be 1 on Kubernetes")
 
-        async with await self._get_storage_client() as storage_client:
-            # extra storage_bucket check is for mypy
-            if storage_client is None or self.storage_bucket is None:
-                raise ValueError(
-                    "storage_bucket and other storage_* parameters must be specified to"
-                    " use Kubernetes with run_map"
-                )
-
+        async with await self.get_storage_bucket() as storage_bucket:
             # pretty much copied from AllocVM.run_map_as_completed
 
-            driver = KubernetesGridJobDriver(self, num_concurrent_tasks, storage_client)
+            driver = KubernetesGridJobDriver(self, num_concurrent_tasks, storage_bucket)
 
             # this should be in get_results, but with indexed workers we need to make
             # sure the tasks are uploaded before we can start workers
@@ -728,12 +671,7 @@ class Kubernetes(Host):
                     storage_key_ranges(driver._job_id),
                 ]
                 await asyncio.gather(
-                    *(
-                        storage_client.delete_object(
-                            Bucket=self.storage_bucket, Key=key
-                        )
-                        for key in keys_to_delete
-                    ),
+                    *(storage_bucket.delete_object(key) for key in keys_to_delete),
                     return_exceptions=True,
                 )
 
@@ -746,18 +684,6 @@ class Kubernetes(Host):
                 f"Received {num_tasks_done}/{len(args)} task results."
             )
 
-    async def get_object_storage(self) -> ObjectStorage:
-        storage_client = await self._get_storage_client()
-
-        if storage_client is None or self.storage_bucket is None:
-            raise ValueError(
-                "Cannot use mirror_local without providing a storage_bucket. Please "
-                "either use a different Deployment method or provide a storage_bucket"
-            )
-        return FuncWorkerClientObjectStorage(
-            await storage_client.__aenter__(), self.storage_bucket
-        )
-
 
 class KubernetesGridJobDriver:
     """
@@ -768,13 +694,13 @@ class KubernetesGridJobDriver:
         self,
         kubernetes: Kubernetes,
         num_concurrent_tasks: int,
-        storage_client: types_aiobotocore_s3.S3Client,
+        storage_bucket: AbstractStorageBucket,
     ):
         self._kubernetes = kubernetes
 
         # properties of the job
         self._num_concurrent_tasks = num_concurrent_tasks
-        self._storage_client = storage_client
+        self._storage_bucket = storage_bucket
 
         self._job_id = str(uuid.uuid4())
         print(f"GridJob id is {self._job_id}")
@@ -798,28 +724,20 @@ class KubernetesGridJobDriver:
     # these three functions are effectively the GridJobCloudInterface
 
     async def _add_tasks(self, args: Sequence[Any]) -> None:
-        assert self._kubernetes.storage_bucket is not None  # just for mypy
-
-        ranges = await upload_task_args(
-            self._storage_client, self._kubernetes.storage_bucket, self._job_id, args
-        )
+        ranges = await upload_task_args(self._storage_bucket, self._job_id, args)
         # this is a hack--"normally" this would get sent with the "task assignment"
         # message, but we don't have the infrastructure for that in the case of Indexed
         # Jobs (static task-to-worker assignment)
-        await self._storage_client.put_object(
-            Bucket=self._kubernetes.storage_bucket,
-            Key=storage_key_ranges(self._job_id),
-            Body=pickle.dumps(ranges),
+        await self._storage_bucket.write_bytes(
+            pickle.dumps(ranges),
+            storage_key_ranges(self._job_id),
         )
 
     async def _receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
     ) -> AsyncIterable[Tuple[List[TaskProcessState], List[WorkerProcessState]]]:
-        assert self._kubernetes.storage_bucket is not None  # just for mypy
-
         return receive_results(
-            self._storage_client,
-            self._kubernetes.storage_bucket,
+            self._storage_bucket,
             self._job_id,
             stop_receiving=stop_receiving,
             all_workers_exited=workers_done,
@@ -838,12 +756,7 @@ class KubernetesGridJobDriver:
         pickle_protocol: int,
     ) -> Job:
 
-        indexed_map_worker_args = (
-            num_args,
-            self._num_concurrent_tasks,
-            self._job_id,
-            pickle.HIGHEST_PROTOCOL,
-        )
+        indexed_map_worker_args = num_args, self._num_concurrent_tasks, self._job_id
 
         return Job(
             job_id=self._job_id,
@@ -885,16 +798,8 @@ class KubernetesGridJobDriver:
 
         async with kubernetes_client.ApiClient() as api_client, kubernetes_stream.WsApiClient() as ws_api_client:  # noqa: E501
             try:
-                storage_endpoint = (
-                    self._kubernetes.get_storage_endpoint_url_in_cluster()
-                )
-
-                if self._kubernetes.storage_bucket is None or storage_endpoint is None:
-                    raise ValueError(
-                        "Cannot use an environment_spec without providing a "
-                        "storage_bucket. Please either provide a pre-built container "
-                        "image for the interpreter or provide a storage_bucket"
-                    )
+                # just for mypy, get_storage_bucket requires this
+                assert self._kubernetes.storage_spec is not None
 
                 job = self._worker_function_job(
                     function, num_args, job_fields, pickle_protocol
@@ -906,10 +811,9 @@ class KubernetesGridJobDriver:
                     modified_job,
                 ) = await _image_name_from_job(job)
 
-                await self._storage_client.put_object(
-                    Bucket=self._kubernetes.storage_bucket,
-                    Key=storage_key_job_to_run(self._job_id),
-                    Body=modified_job.SerializeToString(),
+                await self._storage_bucket.write_bytes(
+                    modified_job.SerializeToString(),
+                    storage_key_job_to_run(self._job_id),
                 )
 
                 core_api = kubernetes_client.CoreV1Api(api_client)
@@ -925,9 +829,7 @@ class KubernetesGridJobDriver:
                     image_pull_secret_name,
                     [int(p) for p in expand_ports(job_fields["ports"])],
                     resources_required_per_task,
-                    storage_endpoint,
-                    self._kubernetes.storage_bucket,
-                    self._kubernetes.storage_username_password_secret,
+                    self._kubernetes.storage_spec,
                     self._job_id,
                     self._num_concurrent_tasks,
                     wait_for_result,
@@ -1007,11 +909,9 @@ class KubernetesGridJobDriver:
                     if worker_launch_task is not None:
                         worker_launch_task.cancel()
 
-                    if self._kubernetes.storage_bucket is not None:
-                        await self._storage_client.delete_object(
-                            Bucket=self._kubernetes.storage_bucket,
-                            Key=storage_key_job_to_run(self._job_id),
-                        )
+                    await self._storage_bucket.delete_object(
+                        storage_key_job_to_run(self._job_id)
+                    )
                 except asyncio.CancelledError:
                     raise
                 except BaseException:
@@ -1164,40 +1064,17 @@ def _pod_meets_requirements(
 def _add_meadowrun_variables_to_environment(
     environment: List[kubernetes_client.V1EnvVar],
     environment_variables: Dict[str, str],
-    storage_username_password_secret: Optional[str],
+    storage_spec: Optional[StorageBucketSpec],
 ) -> None:
     """
     Modifies environment in place!! environment_variables is just to make sure we don't
     overwrite an existing environment variable
     """
 
-    if storage_username_password_secret is not None:
-        if MEADOWRUN_STORAGE_USERNAME not in environment_variables:
-            environment.append(
-                kubernetes_client.V1EnvVar(
-                    name=MEADOWRUN_STORAGE_USERNAME,
-                    value_from=kubernetes_client.V1EnvVarSource(
-                        secret_key_ref=kubernetes_client.V1SecretKeySelector(
-                            key="username",
-                            name=storage_username_password_secret,
-                            optional=False,
-                        )
-                    ),
-                )
-            )
-        if MEADOWRUN_STORAGE_PASSWORD not in environment_variables:
-            environment.append(
-                kubernetes_client.V1EnvVar(
-                    name=MEADOWRUN_STORAGE_PASSWORD,
-                    value_from=kubernetes_client.V1EnvVarSource(
-                        secret_key_ref=kubernetes_client.V1SecretKeySelector(
-                            key="password",
-                            name=storage_username_password_secret,
-                            optional=False,
-                        )
-                    ),
-                )
-            )
+    if storage_spec is not None:
+        for env_var in storage_spec.get_environment_variables():
+            if env_var.name not in environment_variables:
+                environment.append(env_var)
 
     if MEADOWRUN_POD_NAME not in environment_variables:
         environment.append(
@@ -1226,9 +1103,7 @@ class KubernetesRemoteProcesses(abc.ABC):
         image_pull_secret_name: Optional[str],
         ports: List[int],
         resources: Optional[ResourcesInternal],
-        storage_endpoint: str,
-        storage_bucket: str,
-        storage_username_password_secret: Optional[str],
+        storage_spec: StorageBucketSpec,
         job_id: str,
         # TODO this will probably eventually need to be replaced with an explicit list
         # of worker indexes if we want to be able to restart workers
@@ -1287,9 +1162,7 @@ class SingleUsePodRemoteProcesses(KubernetesRemoteProcesses):
         image_pull_secret_name: Optional[str],
         ports: List[int],
         resources: Optional[ResourcesInternal],
-        storage_endpoint: str,
-        storage_bucket: str,
-        storage_username_password_secret: Optional[str],
+        storage_spec: StorageBucketSpec,
         job_id: str,
         num_executions: int,
         wait_for_result: WaitOption,
@@ -1304,13 +1177,9 @@ class SingleUsePodRemoteProcesses(KubernetesRemoteProcesses):
             "python",
             "-m",
             "meadowrun.run_job_local_storage_main",
-            "--storage-bucket",
-            storage_bucket,
             "--job-id",
             job_id,
-            "--storage-endpoint-url",
-            storage_endpoint,
-        ]
+        ] + storage_spec.get_command_line_arguments()
 
         self.run_task = asyncio.create_task(
             _run_kubernetes_job(
@@ -1321,7 +1190,7 @@ class SingleUsePodRemoteProcesses(KubernetesRemoteProcesses):
                 image_name,
                 command,
                 {"PYTHONUNBUFFERED": "1"},
-                storage_username_password_secret,
+                storage_spec,
                 image_pull_secret_name,
                 num_executions,
                 ports,
@@ -1356,7 +1225,7 @@ async def _run_kubernetes_job(
     image: str,
     args: List[str],
     environment_variables: Dict[str, str],
-    storage_username_password_secret: Optional[str],
+    storage_spec: Optional[StorageBucketSpec],
     image_pull_secret_name: Optional[str],
     indexed_completions: Optional[int],
     ports: List[int],
@@ -1388,7 +1257,7 @@ async def _run_kubernetes_job(
     ]
 
     _add_meadowrun_variables_to_environment(
-        environment, environment_variables, storage_username_password_secret
+        environment, environment_variables, storage_spec
     )
 
     if indexed_completions:
@@ -1494,9 +1363,7 @@ class ReusablePodRemoteProcesses(KubernetesRemoteProcesses):
         image_pull_secret_name: Optional[str],
         ports: List[int],
         resources: Optional[ResourcesInternal],
-        storage_endpoint: str,
-        storage_bucket: str,
-        storage_username_password_secret: Optional[str],
+        storage_spec: StorageBucketSpec,
         job_id: str,
         num_executions: int,
         wait_for_result: WaitOption,
@@ -1508,13 +1375,18 @@ class ReusablePodRemoteProcesses(KubernetesRemoteProcesses):
         ] = None,
     ) -> Task[Optional[Sequence[int]]]:
 
+        storage_spec_args = (
+            f"{STORAGE_TYPE} {storage_spec.get_storage_type()} "
+            + " ".join(
+                shlex.quote(arg) for arg in storage_spec.get_command_line_arguments()
+            )
+        )
         # TODO it's possible that a job will run more than once on the same container,
         # although not in parallel, in that case the job log will be overwritten
         _COMMAND_TEMPLATE = (
             "PYTHONUNBUFFERED=1 MEADOWRUN_WORKER_INDEX={worker_index} python -m "
-            "meadowrun.run_job_local_storage_main --storage-bucket {storage_bucket} "
-            "--job-id {job_id} --storage-endpoint-url {storage_endpoint} "
-            f">/var/meadowrun/job_logs/{job_id}.log 2>&1 & echo $$"
+            f"meadowrun.run_job_local_storage_main --job-id {job_id} "
+            f"{storage_spec_args} >/var/meadowrun/job_logs/{job_id}.log 2>&1 & echo $$"
         )
         # echo $$ gets the process id of the current bash session (we will run
         # inner_command in a bash session). We assume that all processes started in this
@@ -1535,7 +1407,7 @@ class ReusablePodRemoteProcesses(KubernetesRemoteProcesses):
             num_executions,
             core_api,
             batch_api,
-            storage_username_password_secret,
+            storage_spec,
             pod_customization,
         ):
 
@@ -1543,12 +1415,7 @@ class ReusablePodRemoteProcesses(KubernetesRemoteProcesses):
                 command = [
                     "/bin/bash",
                     "-c",
-                    _COMMAND_TEMPLATE.format(
-                        worker_index=worker_index,
-                        storage_bucket=storage_bucket,
-                        job_id=job_id,
-                        storage_endpoint=storage_endpoint,
-                    ),
+                    _COMMAND_TEMPLATE.format(worker_index=worker_index),
                 ]
 
                 process_group_id_task = asyncio.create_task(
@@ -1635,7 +1502,7 @@ async def _get_meadowrun_reusable_pods(
     number_of_pods: int,
     core_api: kubernetes_client.CoreV1Api,
     batch_api: kubernetes_client.BatchV1Api,
-    storage_username_password_secret: Optional[str],
+    storage_spec: Optional[StorageBucketSpec],
     pod_customization: Optional[
         Callable[
             [kubernetes_client.V1PodTemplateSpec],
@@ -1696,9 +1563,7 @@ async def _get_meadowrun_reusable_pods(
     if remaining_pods_to_launch > 0:
         environment: List[kubernetes_client.V1EnvVar] = []
 
-        _add_meadowrun_variables_to_environment(
-            environment, {}, storage_username_password_secret
-        )
+        _add_meadowrun_variables_to_environment(environment, {}, storage_spec)
 
         if image_pull_secret_name:
             additional_pod_spec_parameters = {
@@ -1765,12 +1630,20 @@ async def _get_meadowrun_reusable_pods(
 
         await batch_api.create_namespaced_job(kubernetes_namespace, body)
 
-        pods = await get_pods_for_job(
-            core_api,
-            kubernetes_namespace,
-            job_name,
-            [f"{job_name}-{i}-" for i in range(remaining_pods_to_launch)],
-        )
+        try:
+            pods = await get_pods_for_job(
+                core_api,
+                kubernetes_namespace,
+                job_name,
+                [f"{job_name}-{i}-" for i in range(remaining_pods_to_launch)],
+            )
+        except BaseException:
+            # usually the ttl_seconds_after_finished will take care of deleting the job,
+            # but there's a case where the pods can't be created (e.g. if you provide an
+            # invalid serviceAccountName), and the job will stick around forever
+            await batch_api.delete_namespaced_job(job_name, kubernetes_namespace)
+            raise
+
         for pod_future in asyncio.as_completed(
             [
                 wait_for_pod_running(core_api, job_name, kubernetes_namespace, pod)
@@ -1786,3 +1659,6 @@ async def _gather_return_none(tasks: List[Task]) -> None:
     """A workaround because you can't do asyncio.create_task(asyncio.gather(*tasks))"""
     # consider throwing all exceptions not just the first one we get
     await asyncio.gather(*tasks, return_exceptions=False)
+
+
+STORAGE_TYPE = "--storage-type"
