@@ -17,8 +17,6 @@ from decimal import InvalidOperation
 from typing import (
     TYPE_CHECKING,
     Any,
-    AsyncIterable,
-    Callable,
     Coroutine,
     Dict,
     Iterable,
@@ -34,18 +32,6 @@ import psutil
 
 from meadowrun.aws_integration.ecr import get_ecr_username_password
 from meadowrun.azure_integration.acr import get_acr_username_password
-
-if TYPE_CHECKING:
-    from typing_extensions import Literal
-
-    JobSpecType = Literal["py_command", "py_function", "py_agent"]
-
-    from meadowrun._vendor import aiodocker
-
-    from meadowrun._vendor.aiodocker.containers import DockerContainer
-    from meadowrun.instance_selection import ResourcesInternal
-    from meadowrun.object_storage import ObjectStorage
-
 from meadowrun.config import (
     MEADOWRUN_AGENT_PID,
     MEADOWRUN_CODE_MOUNT_LINUX,
@@ -78,16 +64,18 @@ from meadowrun.meadowrun_pb2 import (
     PyFunctionJob,
     StringPair,
 )
-from meadowrun.run_job_core import (
-    CloudProviderType,
-    Host,
-    JobCompletion,
-    MeadowrunException,
-    TaskResult,
-    WaitOption,
-)
 from meadowrun.shared import cancel_task, pickle_exception
 
+if TYPE_CHECKING:
+    from typing_extensions import Literal
+
+    JobSpecType = Literal["py_command", "py_function", "py_agent"]
+
+    from meadowrun._vendor import aiodocker
+    from meadowrun._vendor.aiodocker.containers import DockerContainer
+    from meadowrun.deployment_manager import StorageBucketFactoryType
+    from meadowrun.run_job_core import CloudProviderType
+
 ProcessStateEnum = ProcessState.ProcessStateEnum
 
 _T = TypeVar("_T")
@@ -976,8 +964,7 @@ async def _launch_container_job(
             for (host_path, mounted_path) in binds
             if host_path in code_paths
         )
-        # TODO we need to use ":" for Linux and ";" for Windows containers
-        job_spec_transformed.environment_variables["PYTHONPATH"] = ":".join(
+        job_spec_transformed.environment_variables["PYTHONPATH"] = os.pathsep.join(
             itertools.chain(code_paths, existing_python_path)
         )
 
@@ -1191,20 +1178,17 @@ def _completed_job_state(
 
 def _set_up_working_folder() -> Tuple[str, str, str, str, str]:
     """
-    Sets the working_folder to a default if it's not set, creates the necessary
-    subfolders, gets a machine-wide lock on the working folder, then returns io_folder,
-    job_logs_folder, git_repos_folder, local_copies_folder
+    Returns conventional io, job_logs, git_repos, local_copies, and misc path names.
+    It's assumed that these paths already exist.
     """
 
     working_folder = "/var/meadowrun"
 
-    # first, create the directories that we need
-
     # this holds files for transferring data to and from this server process and the
     # child processes
     io_folder = os.path.join(working_folder, "io")
     # holds the logs for the functions/commands that this server runs
-    job_logs_folder = "/var/meadowrun/job_logs"
+    job_logs_folder = os.path.join(working_folder, "job_logs")
     # see CodeDeploymentManager
     git_repos_folder = os.path.join(working_folder, "git_repos")
     # see CodeDeploymentManager
@@ -1360,6 +1344,7 @@ async def _get_credentials_for_job(
 async def run_local(
     job: Job,
     cloud: Optional[Tuple[CloudProviderType, str]] = None,
+    storage_bucket_factory: StorageBucketFactoryType = None,
     compile_environment_in_container: bool = True,
 ) -> Tuple[ProcessState, Optional[asyncio.Task[ProcessState]]]:
     """
@@ -1411,7 +1396,11 @@ async def run_local(
 
         # first, get the code paths
         code_paths, interpreter_spec_path, cwd_path = await get_code_paths(
-            git_repos_folder, local_copies_folder, job, code_deployment_credentials
+            git_repos_folder,
+            local_copies_folder,
+            job,
+            code_deployment_credentials,
+            storage_bucket_factory,
         )
 
         # next, if we have a environment_spec_in_code, turn into a container
@@ -1620,81 +1609,3 @@ async def run_local(
             ),
             None,
         )
-
-
-@dataclasses.dataclass(frozen=True)
-class LocalHost(Host):
-    async def run_job(
-        self,
-        resources_required: Optional[ResourcesInternal],
-        job: Job,
-        wait_for_result: WaitOption,
-    ) -> JobCompletion[Any]:
-        if wait_for_result != WaitOption.WAIT_SILENTLY:
-            raise NotImplementedError(
-                f"{wait_for_result} is not supported for LocalHost yet"
-            )
-
-        if resources_required is not None:
-            raise ValueError("Specifying Resources for LocalHost is not supported")
-
-        initial_update, continuation = await run_local(job)
-        if (
-            initial_update.state != ProcessState.ProcessStateEnum.RUNNING
-            or continuation is None
-        ):
-            result = initial_update
-        else:
-            result = await continuation
-
-        if result.state == ProcessState.ProcessStateEnum.SUCCEEDED:
-            job_spec_type = job.WhichOneof("job_spec")
-            # we must have a result from functions, in other cases we can optionally
-            # have a result
-            if job_spec_type == "py_function" or result.pickled_result:
-                unpickled_result = pickle.loads(result.pickled_result)
-            else:
-                unpickled_result = None
-
-            return JobCompletion(
-                unpickled_result,
-                result.state,
-                result.log_file_name,
-                result.return_code,
-                "localhost",
-            )
-        else:
-            raise MeadowrunException(result)
-
-    async def run_map(
-        self,
-        function: Callable[[_T], _U],
-        args: Sequence[_T],
-        resources_required_per_task: Optional[ResourcesInternal],
-        job_fields: Dict[str, Any],
-        num_concurrent_tasks: int,
-        pickle_protocol: int,
-        wait_for_result: WaitOption,
-        max_num_tasks_attempts: int,
-        retry_with_more_memory: bool,
-    ) -> Optional[Sequence[_U]]:
-        raise NotImplementedError("run_map on LocalHost is not implemented")
-
-    def run_map_as_completed(
-        self,
-        function: Callable[[_T], _U],
-        args: Sequence[_T],
-        resources_required_per_task: Optional[ResourcesInternal],
-        job_fields: Dict[str, Any],
-        num_concurrent_tasks: int,
-        pickle_protocol: int,
-        wait_for_result: WaitOption,
-        max_num_tasks_attempts: int,
-        retry_with_more_memory: bool,
-    ) -> AsyncIterable[TaskResult[_U]]:
-        raise NotImplementedError(
-            "run_map_as_completed is not implemented for LocalHost"
-        )
-
-    async def get_object_storage(self) -> ObjectStorage:
-        raise NotImplementedError("LocalHost has no object storage")
